use diesel::prelude::*;
use serde::{Serialize, Deserialize};
use crate::{crud::r#trait::CrudEntity, schema::users};

#[derive(Serialize, Deserialize, Queryable, Insertable)]
#[diesel(table_name = users)]
pub struct User {
    id: i32,
    name: String,
    email: String,
}

#[derive(Deserialize, Insertable)]
#[diesel(table_name = users)]
pub struct NewUser {
    name: String,
    email: String,
}

#[derive(Deserialize, AsChangeset)]
#[diesel(table_name = users)]
pub struct UpdateUser {
    name: String,
    email: String,
}

impl CrudEntity for User {
    type Id = i32;
    type CreateInput = NewUser;
    type UpdateInput = UpdateUser; // Changed from User to UpdateUser

    fn find_all(conn: &mut PgConnection, limit: Option<i64>, offset: Option<i64>) -> Result<Vec<Self>, diesel::result::Error> {
        use crate::schema::users::dsl::*;
        let mut query = users.into_boxed();
        if let Some(l) = limit {
            query = query.limit(l);
        }
        if let Some(o) = offset {
            query = query.offset(o);
        }
        query.load::<Self>(conn)
    }

    fn find_by_id(conn: &mut PgConnection, id: Self::Id) -> Result<Self, diesel::result::Error> {
        use crate::schema::users::dsl::*;
        users.filter(id.eq(id)).first::<Self>(conn)
    }

    fn insert(conn: &mut PgConnection, item: Self::CreateInput) -> Result<Self, diesel::result::Error> {
        use crate::schema::users;
        diesel::insert_into(users::table).values(&item).get_result(conn)
    }

    fn update(conn: &mut PgConnection, id: Self::Id, item: Self::UpdateInput) -> Result<Self, diesel::result::Error> {
        use crate::schema::users::dsl::*;
        diesel::update(users.filter(id.eq(id))).set(&item).get_result(conn)
    }

    fn delete(conn: &mut PgConnection, id: Self::Id) -> Result<usize, diesel::result::Error> {
        use crate::schema::users::dsl::*;
        diesel::delete(users.filter(id.eq(id))).execute(conn)
    }
}