use actix_web::{dev::{ServiceFactory, ServiceRequest}, web, App};
use crate::crud::{handlers, r#trait::CrudEntity};

pub fn add_crud_routes<T: CrudEntity + 'static>(mut app: App<T>, path: &str) -> App<T>
where
    T::Id: serde::de::DeserializeOwned + serde::Serialize + 'static,
    T::CreateInput: serde::de::DeserializeOwned + 'static,
    T::UpdateInput: serde::de::DeserializeOwned + 'static,


    T: ServiceFactory<ServiceRequest, Config = (), Error = actix_web::Error, InitError = ()>
{
    app = app.service(
        web::resource(path)
            .route(web::get().to(handlers::get_all::<T>))
            .route(web::post().to(handlers::create::<T>)),
    );
    app.service(
        web::resource(&format!("{}/{{id}}", path))
            .route(web::get().to(handlers::get_one::<T>))
            .route(web::put().to(handlers::update::<T>))
            .route(web::delete().to(handlers::delete::<T>)),
    )
}